// For licensing, see: http://hydrogen.knownspace.org/license.html
package org.datamanager.constraint;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;

import org.datamanager.event.AddEntityPoolEvent;
import org.datamanager.event.DataManagerEvent;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.AbstractEventConstraint;
import org.datamanager.kernel.Entity;

/**
 * This EventConstraint describes a subset of events that
 * contain Entities that are described by a given
 * EntityConstraint. This EventConstraint is only useful
 * when subscribed to an EventGenerator that produces
 * AddEntityPoolEvents.
 *
 * @author Matthew Farrellee
 */
public class EventContainsEntityConstraint
    extends AbstractEventConstraint
    {
    /**
     * The constraint to filter out all AddEntityPoolEvents
     */
    private static final EventConstraint eventConstraint = 
            new EventTypeIsConstraint(AddEntityPoolEvent.class);
    /**
     * The constraint to filter out all newly added Entities
     * stored in an AddEntityPoolEvent.
     */
    private EntityConstraint entityConstraint;

    /**
     * Create a new EventConstainsEntityConstraint to check
     * for Entities that satisfy the given EntityConstraint.
     *
     * @param constraint An EntityConstraint to use when filtering
     *                   newly added entities.
     */
    public EventContainsEntityConstraint(EntityConstraint constraint)
        {
        this.entityConstraint = constraint;
        }

    /**
     * Check whether the event is an AddEntityPoolEvent and the
     * new Entity in the event satisfies the constraint
     * given in the constructor of this constraint.
     *
     * @param event An event to check
     *
     * @return Whether the event is an AddEntityPoolEvent and the
     *         new Entity in the event satisfies the constraint
     *         given in the constructor of this constraint.
     */
    protected boolean describes(DataManagerEvent event)
                         throws DataManagerException
        {
        if (eventConstraint.accepts(event))
            {
            Entity entity = ((AddEntityPoolEvent) event).getAddedEntity();

            return entityConstraint.accepts(entity);
            }
        else
            {
            return false;
            }
        }
    }