// For licensing, see: http://hydrogen.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
// 2002-10-11 baa
// Fixed a few typos/misspelings in comments.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-03 cam
// Fixed a typo in a comment.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-03 cam
// Added getKernelWindow() and changed to pass SimpletonList from
// SimpletonLoaderProcess to KernelWindow.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-05 cam
// Changed to use KernelGui instead of KernelWindow.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-12 nab
// - added startPoolPersistingThread() method to persist the pool and entities
//   on a regular interval and call the method in main()
// - add a constant SAVE_INTERVAL_IN_MINUTES for the thread persist interval
// - removed the System.runFinalizersOnExit(true) call
///////////////////////////////////////////////////////////////////////////////
// 2002-10-14 cam/cjd
// 1. Turned the persistence by 1 minute intervals back on.
// 2. Made the persist call be to the EntityManager, not the pool.
///////////////////////////////////////////////////////////////////////////////

/**
 * DataManager.java
 */
package org.datamanager.kernel;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.KernelGui;
import org.datamanager.kernel.SimpletonLoaderProcess;

import org.datamanager.tools.Debug;

/**
 * This is the startup code for the datamanager. When invoked this
 * class will load a KernelGui that can be used to run
 * Simpletons and UserInterfaces that are located in the startup
 * directory.
 *
 * @author Matthew Farrellee
 */
public class DataManager
    {
    /**
     * The startup directory's name.
     */
    public static final String STARTUP = "startup";
    /**
     * The startup data XML file's name.
     */
    public static final String STARTUP_DATA = "knownspace_config.xml";
    /**
     * The simpleton directory's name.
     */
    public static final String SIMPLETON = "simpletons";
    /**
     * The name a userinterface's startup simpleton must end with.
     */
    public static final String USER_INTERFACE_NAME = "InterfaceStartupSimpleton";
    /**
     * A File pointing to the startup data XML.
     */
    public static final File STARTUP_DATA_FILE = new File(STARTUP_DATA);
    /**
     * The name of the files where the default pool will be
     * serialized.
     */
    public static final String DEFAULT_POOL = "DefaultPool";
    /**
     * The name of the file where all the entities in the
     * system will be serialized.
     */
    public static final String OBJECT_STORE = "ObjectStore";
    /**
     *
     */
    private static final int SAVE_INTERVAL_IN_MINUTES = 1;
    /**
     * The GUI used to start things in the DataManager.
     */
    private KernelGui controller;

    /**
     * Private default constructor.  Constructs the default KernelGui and
     * delegates to the DataManager(KernelGui controller) constructor (also
     * private).
     */
    private DataManager()
        {
        this(new KernelGui());
        }

    /**
     * Private constructor parameterizing an externally instantiated
     * KernelGui.
     *
     * @param controller The GUI used to start things in
     *                   the DataManager.
     */
    private DataManager(KernelGui controller)
        {
        this.controller = controller;
        }

    /**
     * Starts the SimpletonLoaderProcess.
     */
    protected void startSimpletonLoader()
        {
        // This process loads all Simpletons.
        SimpletonLoaderProcess loader = new SimpletonLoaderProcess(
                                                STARTUP_DATA_FILE);

        controller.setSimpletonList(loader.getSimpletonList());

        loader.start();

        // Tries to set the priority.  Being unsuccessful currently does
        // nothing but prints a stack trace (forgets about the failure).  This
        // should probably be fixed.
        try
            {
            loader.setPriority(Simpleton.Priority.LOW_PRIORITY);
            }
        catch (DataManagerException exception)
            {
            exception.printStackTrace();
            }
        }

    /**
     * Returns the KernelGui controller.
     */
    KernelGui getKernelGui()
        {
        return controller;
        }

    /**
     * Displays the KernelGui conroller.
     */
    private void display()
        {
        controller.display();
        }

    /**
     * This will start up a thread that will run every one minute.  This thread
     * will persist the pool, and all of its entities.
     */
    private static void startPoolPersistingThread()
        {
        new Thread(new Runnable()
            {
            public void run()
                {
                while (true)
                    {
                    try
                        {
                        Debug.writeLogMessage("*** Sleeping.");
                        Thread.sleep(1000 * 60 * SAVE_INTERVAL_IN_MINUTES);
                        Debug.writeLogMessage("*** starting to persist pool.");
                        EntityManager.getInstance()
                                     .persist(VerySimpleEntityManager.PERSISTENCE_FILE);
                        Debug.writeLogMessage("*** finished persisting pool.");
                        }
                    catch (Exception exception)
                        {
                        Debug.writeLogMessage("persisting the EntityManager" + " and Pool failed " + 
                                                  exception.getMessage());
                        }
                    }
                }
            }).start();
        }

    /**
     * Main method. Currently it does not take any parameters.
     */
    public static void main(String[] input)
        {
        startPoolPersistingThread();

        // Stuff to keep the console from being spammed.
        //File output = new File("output");
        //PrintStream fOut = new PrintStream(new FileOutputStream(output));
        //System.setErr(fOut);
        //System.setOut(fOut);
        // Create a DataManager.
        DataManager dataManager = new DataManager();

        // Sets up the debugger.
        Debug.setDebugger(true);

        // Displays the KernelGui.
        dataManager.display();

        // Starts the Kernel Processes
        dataManager.startSimpletonLoader();
        }
    }