// For licensing, see: http://hydrogen.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
// 2002-10-11 baa
// Tidied comments/javadoc.  Renamed reset(Simpleton simpleton) to
// setSimpleton(Simpleton simpleton).
///////////////////////////////////////////////////////////////////////////////
// 2002-10-05 cam
// Moved translatePriority() to Simpleton.Priority.
// Tidied comments/javadoc.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.Simpleton;

import org.datamanager.tools.Debug;

/**
 * This class is the parent of all threads that are run
 * within the kernel and can work on Entities. It has
 * a special switch for events. This allows the kernel
 * to know when it should fire events from Entities. For
 * example, when a Constraint is working on an Entity it
 * should not fire an event, so the switch is turned off
 * while the Thread is executing within the Constraint,
 * and turned back on when it leaves the Constraint. This
 * is done from within the AbstractConstraint classes.
 *
 * @author Matthew Farrellee
 */
class DataManagerWorker
    extends Thread
    {
    /** Determines if an event should be fired or not. */
    private boolean generateEvent;
    /** The target Simpleton that this Thread will run. */
    private Simpleton simpleton;

    /**
     * Creates a DataManagerWorker that will run the given
     * Simpleton.
     */
    public DataManagerWorker(Simpleton simpleton)
        {
        super();
        setSimpleton(simpleton);
        generateEvents(true);
        }

    /**
     * Starts this DataManagerWorker.
     * NOTE:
     * This is currently useless.  Unless there are provisions in the future
     * to have extra functionality beyond that of the super Thread's start
     * method, this extra level of indirection is unnecessary.  -- BAA 
     * @param none
     */
    public void start()
        {
        super.start();
        }

    /**
     * Runs this DataManagerWorker by invoking the simpleton's process()
     * method.
     * @see Simpleton.process(), Thread.run()
     */
    public void run()
        {
        simpleton.process();
        }

    /**
     * Sets the DataManagerWorker to a different
     * Simpleton. This is the Simpleton whose
     * process method will be executed when the
     * DataManagerWorker is started.  
     * 
     * @param simpleton 
     */
    protected void setSimpleton(Simpleton simpleton)
        {
        this.simpleton = simpleton;
        }

    /**
     * Sets the priority of this Thread using the
     * Simpleton.Priority class.
     */
    public void setPriority(Simpleton.Priority priority)
        {
        super.setPriority(priority.translateToThreadPriority());
        }

    /**
     * Gets the Simpleton.Priority of this DataManagerWorker.
     * @param none
     * @return Simpleton.Priority 
     */
    public Simpleton.Priority getSimpletonPriority()
        {
        // Get the Thread's priority.
        int threadPriority = getPriority();

        return Simpleton.Priority.translateToSimpletonPriority(threadPriority);
        }

    /**
     * Sets the event generation switch.  Kernel access only, hence the lack
     * of a public/protected/private modifier.
     *
     * @param generateEvent boolean specifying whether or not this thread
     * should cause events to be fired.
     */
    void generateEvents(boolean generateEvent)
        {
        this.generateEvent = generateEvent;
        }

    /**
     * Tests to see if this thread should cause an event.  Kernel access only,
     * hence the lack of a public/protected/private modifier.
     * @param none
     * @return boolean 
     */
    boolean isEventGenerating()
        {
        return generateEvent;
        }
    }