// For licensing, see: http://hydrogen.knownspace.org/license.html
///////////////////////////////////////////////////////////////////////////////
//EntityManager.java
//Created: 09/28/1999 08:24
///////////////////////////////////////////////////////////////////////////////
//Modified (10/06/2002): maa
//1) Cleaned up JavaDoc to adhere to standards and eliminate conflicts.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-10 cam
// Cleaned up javadoc a little more.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-13 baa
// Added initial integration with persistence layer and modified some
// javadocs.  Added abstract persist(File file) and default-implemented
// restore(File file) methods. 
///////////////////////////////////////////////////////////////////////////////
// 2002-10-13 cam/cjd
// Turned on serializable style of persistence, rather than XML,
// as that works
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.io.File;
import java.io.Serializable;

import org.datamanager.kernel.Persistable;
import org.datamanager.kernel.Pool;

import org.datamanager.tools.Debug;

/**
 * The EntityManager is responsible for maintaining lookup tables between
 * the system IDs, the EntityProxies, and the actually entities they proxy
 * for.  Ideally, an EntityManager should also cache unused Entities to
 * disk and perform other such optomizations.
 *
 * @author Matthew Farrellee
 * @author Benjamin Keil
 */
abstract class EntityManager
    implements Serializable
    {
    /**
     * Requests the instance of the EntityManager.
     *
     * @return The instance of the EntityManager.
     */
    static EntityManager getInstance()
        {
        return VerySimpleEntityManager.getInstance();
        }

    /**
     * Creates a new Entity with the given name.
     *
     * @param String The name of the new Entity.
     * @return A new EntityProxy pointing to a new Entity with the given name.
     */
    protected abstract EntityProxy createEntity(String name);

    /**
     * Finds the EntityProxy that has the given identifier.
     *
     * @param Identifier The Identifier for which to find the EntityProxy.
     * @return The EntityProxy with the given Identifier.
     */
    protected abstract EntityProxy findProxy(Identifier identifier);

    /**
     * Finds the Entity that has the given identifier.
     *
     * @param Identifier The Identifier for which to find the Entity.
     * @return The Entity with the given Identifier.
     */
    protected abstract Entity findEntity(Identifier identifier);

    /**
     * Finds the Identifier that identifies the given EntityProxy.
     *
     * @param EntityProxy The EntityProxy for which to find the Identifier.
     * @return The Identifier for the given EntityProxy.
     */
    protected abstract Identifier findIdentifier(EntityProxy entityProxy);

    /**
     * Finds the Identifier that identifies the given entity.
     *
     * @param Entity The Entity for which to find the Identifier.
     * @return The Identifier for the given entity.
     */
    protected abstract Identifier findIdentifier(Entity entity);

    /**
     * Persists this EntityManager to the given outputFile
     *
     * @param File outputFile to which this EntityManager is to be written.
     */
    protected abstract void persist(File outputFile);

    /**
     * Restores the EntityManager specified by the given inputFile
     *
     * @param File inputFile from which an EntityManager is to be restored.
     * @return EntityManager restored from the file
     */
    protected static EntityManager restore(File inputFile)
        {
        return (EntityManager) Persistable.PERSISTER.restore(inputFile);
        }

    /** Gets the Pool associated with this EntityManager. */
    protected abstract Pool getPool();
    }