// For licensing, see: http://hydrogen.knownspace.org/license.html
//////////////////////////////////////////////////////////////////////////////
// 2002-10-11 baa
// We are capable of the simplest of tasks!  Renamed reset() method to
// resetSimpleton() in DataManagerWorker and propagated that change to the
// Simpleton start() method.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-05 cam
// Added translateToThreadPriority() and translateToSimpletonPriority(), moved
// from translatePriority() on DataManagerWorker.
// Cleaned up comments.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// Changed Debug.assert() calls to Debug.writeLogMessage() to enable
// java 1.4 assertions elsewhere.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// Simpleton now implements Cloneable, so that the GUI can create multiple
// copies of any Simpleton.
//
// This is potentially the first step.  The next step might be to allow a
// Simpleton to specify (in its Simpleton.Information) whether or not it should
// be a single-instance-only Simpleton and whether or not it should be
// restartable once it finishes.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.DataManagerWorker;

import org.datamanager.tools.Debug;

/**
 * A generic process to implement arbitrary entity and
 * event manipulation.
 * 
 * @author Matthew Farrellee
 */
public abstract class Simpleton
    implements Cloneable
    {
    /** The thread that will run the Simpleton. */
    private DataManagerWorker thread;
    /** The Information about the Simpleton. */
    private Simpleton.Information information;

    /**
     * Creates a new Simpleton with Information set
     * to Simpleton.Information.NONE.
     */
    public Simpleton()
        {
        this.information = Simpleton.Information.NONE;

        thread = new DataManagerWorker(this);
        }

    /**
     * Sets the priority of this Simpleton.
     */
    public void setPriority(Simpleton.Priority priority)
                     throws DataManagerException
        {
        thread.setPriority(priority);
        }

    /**
     * Gets the priority of this Simpleton.
     */
    public Simpleton.Priority getPriority()
                                   throws DataManagerException
        {
        return thread.getSimpletonPriority();
        }

    /**
     * Starts the Simpleton (with a possibly redundant setSimpleton invocation
     * on this Simpleton's DatamanagerWorker thread).
     */
    public void start()
        {
        thread.setSimpleton(this);
        thread.start();
        }

    /**
     * Sets the Information object for this Simpleton.
     */
    protected void setInformation(Simpleton.Information information)
        {
        this.information = information;
        }

    /**
     * Reports Simpleton.Information about this Simpleton.
     */
    public Simpleton.Information getInformation()
        {
        return information;
        }

    /**
     * Performs the work of this Simpleton. Simpleton
     * implementation must override this class.
     */
    public abstract void process();

    /**
     * Clones this Simpleton.  Subclasses of Simpleton should override this
     * method, calling super.clone() first.  The subclass clone() method should
     * make sure that any reference variables in the clone are handled
     * appropriately.  You may want to make copies of the referred-to object
     * rather than point the clone at the same object as the original.
     *
     * @see java.lang.Object#clone()
     */
    protected Object clone()
                    throws CloneNotSupportedException
        {
        Simpleton clonedSimpleton = (Simpleton) super.clone();

        clonedSimpleton.thread = new DataManagerWorker(clonedSimpleton);

        return clonedSimpleton;
        }

    /////////////////////////
    // Inner Classes Below //
    /////////////////////////

    /**
     * Encapsulates a Priority that can be used to control
     * thread priority.
     */
    public static final class Priority
        {
        public final static Priority MIN_PRIORITY = new Priority();
        public final static Priority LOW_PRIORITY = new Priority();
        public final static Priority NORMAL_PRIORITY = new Priority();
        public final static Priority HIGH_PRIORITY = new Priority();
        public final static Priority MAX_PRIORITY = new Priority();

        private Priority()
            {
            }

        /**
         * Translates this Simpleton.Priority to a Thread priority.
         */
        public int translateToThreadPriority()
            {
            // The default, incase the Simpleton.Priority given is
            // unknown, is Thread.MIN_PRIORITY.
            int threadPriority = Thread.MIN_PRIORITY;

            // Translate the Simpleton.Priority into a priority that
            // java.lang.Thread can understand.
            if (this == Simpleton.Priority.MAX_PRIORITY)
                {
                threadPriority = Thread.MAX_PRIORITY;
                }
            else if (this == Simpleton.Priority.HIGH_PRIORITY)
                {
                threadPriority = Thread.NORM_PRIORITY + 2;
                }
            else if (this == Simpleton.Priority.NORMAL_PRIORITY)
                {
                threadPriority = Thread.NORM_PRIORITY;
                }
            else if (this == Simpleton.Priority.LOW_PRIORITY)
                {
                threadPriority = Thread.MIN_PRIORITY + 2;
                }
            else if (this == Simpleton.Priority.MIN_PRIORITY)
                {
                threadPriority = Thread.MIN_PRIORITY;
                }
            else
                {
                Debug.writeLogMessage("Unknown Priority Given.");
                }

            return threadPriority;
            }

        /**
         * Translates an int priority from the Thread class to
         * a Simpleton.Priority.
         */
        public static Priority translateToSimpletonPriority(int priority)
            {
            // The default, incase the priority given is
            // unknown, is MIN_PRIORITY.
            Simpleton.Priority simpletonPriority = Simpleton.Priority.MIN_PRIORITY;

            // Translate the given priority into a
            // Simpleton.Priority.
            if (priority == Thread.MAX_PRIORITY)
                {
                simpletonPriority = Simpleton.Priority.MAX_PRIORITY;
                }
            else if (priority == Thread.NORM_PRIORITY)
                {
                simpletonPriority = Simpleton.Priority.NORMAL_PRIORITY;
                }
            else if (priority == Thread.MIN_PRIORITY)
                {
                simpletonPriority = Simpleton.Priority.MIN_PRIORITY;
                }
            else if ((priority > Thread.MIN_PRIORITY) && 
                         (priority < Thread.NORM_PRIORITY))
                {
                simpletonPriority = Simpleton.Priority.LOW_PRIORITY;
                }
            else if ((priority > Thread.NORM_PRIORITY) && 
                         (priority < Thread.MAX_PRIORITY))
                {
                simpletonPriority = Simpleton.Priority.HIGH_PRIORITY;
                }
            else
                {
                Debug.writeLogMessage("Given an unknown priority.");
                }

            return simpletonPriority;
            }

        public String toString()
            {
            String string = "Priority.UNKNOWN";

            if (this == Priority.MIN_PRIORITY)
                {
                string = "Priority.MIN_PRIORITY";
                }
            else if (this == Priority.LOW_PRIORITY)
                {
                string = "Priority.LOW_PRIORITY";
                }
            else if (this == Priority.NORMAL_PRIORITY)
                {
                string = "Priority.NORMAL_PRIORITY";
                }
            else if (this == Priority.HIGH_PRIORITY)
                {
                string = "Priority.HIGH_PRIORITY";
                }
            else if (this == Priority.MAX_PRIORITY)
                {
                string = "Priority.MAX_PRIORITY";
                }

            return string;
            }
        }

    /**
     * Encapsulates information that describes a Simpleton.
     *
     * @author Matthew Farrellee
     */
    public static class Information
        {
        /**
         * A default Information object used when a
         * hacker does not want to bother with making
         * a specific Information object.
         */
        public static final Information NONE;

        static
            {
            String[] authors = {"Nameless Simpleton Hacker"};
            NONE = new Information("java.lang.Object", "John \"Simpleton\" Doe", 
                                   "A poor homeless Simpleton.", 
                                   "A Simpleton that lacks a home " + 
                                   "and identity.", 
                                   "Something greater than Zero.", authors);
            }

        /** The class name of this component. */
        private String className;
        /** The name of this component. */
        private String name;
        /** A short description of this component. */
        private String shortDescription;
        /** A long description of this component. */
        private String longDescription;
        /** The version of this component. */
        private String version;
        /** The authors of this component. */
        private String[] authors;

        /**
         * Creates a new Information object.
         */
        public Information(String className, String name, 
                           String shortDescription, String longDescription, 
                           String version, String[] authors)
            {
            this.className = className;
            this.name = name;
            this.shortDescription = shortDescription;
            this.longDescription = longDescription;
            this.version = version;
            this.authors = authors;
            }

        /**
         * Gets the class name of this Simpleton.
         */
        public String getClassName()
            {
            return className;
            }

        /**
         * Gets the name of this Simpleton.
         */
        public String getName()
            {
            return name;
            }

        /**
         * Gets the short description of this Simpleton.
         */
        public String getShortDescription()
            {
            return shortDescription;
            }

        /**
         * Gets the long description of this Simpleton.
         */
        public String getLongDescription()
            {
            return longDescription;
            }

        /**
         * Gets the version of this Simpleton.
         */
        public String getVersion()
            {
            return version;
            }

        /**
         * Gets the names of the authors of this Simpleton.
         */
        public String[] getAuthors()
            {
            return authors;
            }
        }
    }