///////////////////////////////////////////////////////////////////////////////
// 2002-10-11 baa
// Added param/return to javadocs and updated some javadocs.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-03 cam
// File created.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-07 cam
// Changed from Vector to ArrayList, since it's more lightweight and we're now
// handling synchronization on our own.
//
// Modified so that concurrent access from multiple threads is no longer an
// issue.  This means two things:
//
// 1. All methods that access the underlying List are synchronized.
// 2. iterator() now returns an Iterator over a temporary copy of the underlying
//    list, so there is no risk of either of the following:
//
//    a. Callers modifying the underlying list through method calls on the
//       Iterator.
//    b. The Iterator becoming invalid because the list has been manipulated by
//       other threads.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

/**
 * SimpletonList holds a list of Simpletons used
 * by the SimpletonLoaderProcess and can be 
 * observed by objects that need to know of the list,
 * including the KernelGui.
 *
 * @author Chris Dent
 * @author Matt Liggett
 */
public class SimpletonList
    extends Observable
    {
    /** The list that stores the simpletons */
    private List simpletonList = new ArrayList();

    /**
     * Adds a Simpleton to this list and notifies all SimpletonList observers
     * of the addition.
     *
     * @param simpleton Simpleton to be added to the list of Simpletons.
     */
    public synchronized void add(Simpleton simpleton)
        {
        simpletonList.add(simpleton);
        notifyOfChange();
        }

    /**
     * Removes a Simpleton from this list and notifies all SimpletonList
     * observers of the change.
     *
     * @param simpleton Simpleton to be removed. 
     */
    public synchronized void remove(Simpleton simpleton)
        {
        simpletonList.remove(simpleton);
        notifyOfChange();
        }

    /**
     * Gets the size of this list.
     * @param none
     * @return int representing the number of simpletons in this SimpletonList
     */
    public synchronized int size()
        {
        return simpletonList.size();
        }

    /**
     * Gets the simpleton at the given index.
     * @param index int index into the SimpletonList
     * @return Simpleton at the given index.
     */
    public synchronized Simpleton get(int index)
        {
        return (Simpleton) simpletonList.get(index);
        }

    /**
     * Returns an Iterator over a copy of the list.
     * @param none
     * @return Iterator over the SimpletonList.  
     * @see java.util.Iterator
     */
    public synchronized Iterator iterator()
        {
        // First make a copy of the list, then return an iterator over that.
        // By doing this, we avoid any synchronization issues with the
        // original, private simpletonList.
        List listCopy = new ArrayList();
        listCopy.addAll(simpletonList);

        return listCopy.iterator();
        }

    /**
     * Notifies the observers of a change in the list.
     */
    private void notifyOfChange()
        {
        setChanged();
        notifyObservers();
        clearChanged();
        }
    }