// For licensing, see: http://hydrogen.knownspace.org/license.html
//
//VerySimpleEntity.java
//Created: 11/19/1999 15:00
//
//Modified (10/05/2002): maa
//1) Removed redundant implementation of Serializable.
//2) Changed declaration of the attributes and bases instance variables from
//Vector to List (program to interface, not implementation).  We would like to
//create our own EntityList, but did not consider this a priority right now.
//3) Changed getEntities() and chooseEntities() methods to take Lists, rather
//than Vectors, and renamed the parameter of getEntities() from vector to entities.
//4) Modified the following methods that were using Debug.assert() to indicate
//that the class did not support to throw UnsuportedOperationException:  attach(),
//detach(), subscribe(), unsubscribe().
//
//Modified (10/06/2002): maa
//1) Modified all of the methods that were using Debug.assert() for null checks
//on params passed in to throw an IllegalArgumentException if the param is null.
//2) Renamed variables in getEntities() method for clarity.
//3) Changed temp reference variable accepted in chooseEntities() from Vector to List.
//4) Replaced defendant temp variable in chooseEntities() with query.
//5) Replaced conditional with method for contraint accepts test in chooseEntities() method.
//6) Renamed store parameter to candidateEntityIdentifiers and accepted List to selectedEntityProxies in chooseEntities() method.
//7) Changed the scope of chooseEntities() from protected to private.
//8) Put in @todos for logging.
//9) Removed IllegalArgumentException throws from all methods.  We would like to
//replace with NullObjects whereever possible.  We did just that for setValue().
//
//Modified (10/08/2002): maa
//1) Logged everywhere that we are throwing an IllegalArgumentException.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-10 cam
// 1. Cleaned up javadoc.
// 2. Removed duplicated null parameter checks from getAttributes() and
//    getBases() -- we were making the same check in chooseEntities().
// 3. Removed unneeded null parameter check from chooseEntities() -- we always
//    initialize the Lists they are called with in the constructor.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.io.Serializable;

import java.util.List;
import java.util.Vector;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;

import org.datamanager.exception.DataManagerException;

import org.datamanager.kernel.EntityValue;

import org.datamanager.passiveentityvalue.NullEntityValue;

import org.datamanager.tools.Debug;

/**
 * A very simple Entity implementation that is always in memory.  They are
 * never cached on disk.
 *
 * @see org.datamanager.kernel.VerySimpleEntityManager
 *
 * @author Matthew Farrellee
 * @author Benjamin Keil
 */
class VerySimpleEntity
    extends Entity
    {
    /** This Entity's name. */
    private String name;
    /** This Entity's value. */
    private EntityValue value;
    /** This Entity's list of attributes. */
    private List attributes;
    /** This Entity's list of bases. */
    private List bases;

    /**
     * Creates an Entity with the given name.
     *
     * @param name The new VerySimpleEntity's name.
     */
    protected VerySimpleEntity(String name)
        {
        this.name = name;
        value = NullEntityValue.getInstance();
        attributes = new Vector(5, 5);
        bases = new Vector(5, 5);
        }

    /**
     * Attaches the given Entity as an attribute of this Entity.
     *
     * @param attribute The Entity to attach.
     * @throws DataManagerException
     */
    public void attach(Entity attribute)
                throws DataManagerException
        {
        throw new UnsupportedOperationException(
                "attach() was called on VerySimpleEntity, which does not support this method!");
        }

    /**
     * Detaches the given Entity as an attribute of this Entity.
     *
     * @param attribute The Entity to detach.
     * @throws DataManagerException
     */
    public void detach(Entity attribute)
                throws DataManagerException
        {
        throw new UnsupportedOperationException(
                "detach() was called on VerySimpleEntity, which does not support this method!");
        }

    /**
     * Attaches the Entity associated with the given Identifier as an attribute
     * of this Entity.  Relies on a private method call to make a link back
     * from the attribute.
     *
     * @param attribute The Identifier of the entity to be added as an attribute.
     * @throws DataManagerException
     */
    public void attach(Identifier attribute)
                throws DataManagerException
        {
        if (attribute == null)
            {
            Debug.writeLogMessage(
                    "Trying to attach attribute Entity with null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to attach attribute Entity with null Identifier!");
            }
        synchronized (attributes)
            {
            if (bases.contains(attribute))
                {
                throw new DataManagerException(attribute + 
                                               " already attached as base!");
                }

            if (!attributes.contains(attribute))
                {
                attributes.add(attribute);
                }
            }
        }

    /**
     * Adds the Entity identified by the given Identifier to the list of bases.
     * This should only be called from the attach() method of the
     * <code>base</code>.  That is, when it has just added this EntityProxy as
     * an attribute of itself.
     *
     * @param base The Identifier of the base to be added.
     */
    protected void addBase(Identifier base)
        {
        if (base == null)
            {
            Debug.writeLogMessage(
                    "Trying to add base Entity with null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to add base Entity with null Identifier!");
            }
        synchronized (bases)
            {
            bases.add(base);
            }
        }

    /**
     * Detaches the Entity identified by the given Identifier from this Entity.
     *
     * @param attribute The Identifier of the attribute to be added.
     * @throws DataManagerException
     */
    public void detach(Identifier attribute)
                throws DataManagerException
        {
        if (attribute == null)
            {
            Debug.writeLogMessage(
                    "Trying to detach attribute Entity with null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to detach attribute Entity with null Identifier!");
            }
        synchronized (attributes)
            {
            if (attributes.contains(attribute))
                {
                attributes.remove(attribute);
                }
            }
        }

    /**
     * Removes an Entity from this Entity's list of bases.  This should only be called
     * by the base as it removes this Entity as an attribute (which it does
     * automatically), that is, you should probably *NEVER* write a call to
     * this method explicitly.
     *
     * @param base The Identifier of the base to be removed.
     */
    protected void removeBase(Identifier base)
        {
        if (base == null)
            {
            Debug.writeLogMessage(
                    "Trying to remove base Entity with null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to remove base Entity with null Identifier!");
            }
        synchronized (bases)
            {
            bases.remove(base);
            }
        }

    /**
     * Requests this Entity's name.
     *
     * @return The Entity's name.
     * @throws DataManagerException
     */
    public String getName()
                   throws DataManagerException
        {
        return name;
        }

    /**
     * Requests this Entity's value.
     *
     * @return The Entity's value.
     * @throws DataManagerException
     */
    public EntityValue getValue()
                         throws DataManagerException
        {
        return value;
        }

    /**
     * Sets this Entity's value.
     *
     * @param newValue The Entity's new value.
     * @throws DataManagerException
     */
    public void setValue(EntityValue newValue)
                  throws DataManagerException
        {
        synchronized (value)
            {
            if (newValue == null)
                {
                this.value = NullEntityValue.getInstance();
                }
            else
                {
                this.value = newValue;
                }
            }
        }

    /**
     * Gets a constrained array of all entities linked to this Entity.
     *
     * @param constraint The EntityConstraint to filter attributes with.
     * @return An array of attributes that passed the constraint.
     * @throws DataManagerException
     */
    public Entity[] getAttributes(EntityConstraint constraint)
                           throws DataManagerException
        {
        synchronized (attributes)
            {
            return chooseEntities(constraint, attributes);
            }
        }

    /**
     * Gets a constrained array of all entities of which this Entity is an
     * attribute.
     *
     * @param constraint The EntityConstraint to filter bases with.
     * @return bases An array of bases that passed the constraint.
     * @throws DataManagerException
     */
    public Entity[] getBases(EntityConstraint constraint)
                      throws DataManagerException
        {
        synchronized (bases)
            {
            return chooseEntities(constraint, bases);
            }
        }

    /**
     * Always throws UnsupportedOperationException, because this functionality is handled in EntityProxy.
     *
     * @see org.datamanager.kernel.EntityProxy
     * @throws DataManagerException
     */
    public void subscribe(EventHandler listener, EventConstraint constraint)
                   throws DataManagerException
        {
        throw new UnsupportedOperationException(
                "subscribe() was called on VerySimpleEntity, which does not support this method!");
        }

    /**
     * Always throws UnsupportedOperationException, because this functionality is handled in EntityProxy.
     *
     * @see org.datamanager.kernel.EntityProxy
     * @throws DataManagerException
     */
    public void unsubscribe(EventHandler handler)
                     throws DataManagerException
        {
        throw new UnsupportedOperationException(
                "unsubscribe() was called on VerySimpleEntity, which does not support this method!");
        }

    /**
     * Returns an array of all the Entities in the given List that satisfy the
     * given Constraint.
     *
     * @param constraint An EntityConstraint to filter the entities with.
     * @param candidateEntityIdentifiers A List of Identifiers for candidate Entitys
     *                                   to chose from.
     * @return An array of EntityProxys that were identified in the
     *         candidateEntityIdentifiers List and were accepted by the
     *         EntityConstraint.
     * @throws DataManagerException
     */
    private Entity[] chooseEntities(EntityConstraint constraint, 
                                    List candidateEntityIdentifiers)
                             throws DataManagerException
        {
        if (constraint == null)
            {
            Debug.writeLogMessage(
                    "Trying to choose Entitys with null EntityConstraint!");
            throw new IllegalArgumentException(
                    "Trying to choose Entitys with null EntityConstraint!");
            }

        List selectedEntityProxies = new Vector();

        for (int i = 0; i < candidateEntityIdentifiers.size(); i++)
            {
            if (constraintAcceptsEntity(constraint, 
                                        (Identifier) candidateEntityIdentifiers.get(i)))
                {
                selectedEntityProxies.add(EntityManager.getInstance()
                                                       .findProxy((Identifier) candidateEntityIdentifiers.get(i)));
                }
            }

        return getEntities(selectedEntityProxies);
        }

    /**
     * Returns true if the EntityConstraint accepts the Entity associated with
     * the Identifier; false if not.
     *
     * @param constraint An EntityConstraint to test the Entity associated with
     *                  the Identifier against.
     * @param entityIdentifier An Identifier for the Entity that we wish to test.
     * @return A boolean indicating whether or not the EntityConstraint accepts
     *         the Entity associated with the Identifier.
     * @throws DataManagerException
     */
    private boolean constraintAcceptsEntity(EntityConstraint constraint, 
                                            Identifier entityIdentifier)
                                     throws DataManagerException
        {
        return constraint.accepts(EntityManager.getInstance()
                                               .findProxy(entityIdentifier));
        }

    /**
     * Turns a List of Entities into an Array of Entities.
     *
     * @param entityList A List of Entities to turn into an array.
     * @return entityArray An array of Entities.
     */
    protected Entity[] getEntities(List entityList)
        {
        if (entityList == null)
            {
            Debug.writeLogMessage(
                    "Trying to turn null List of Entitys into Entity[]!");
            throw new IllegalArgumentException(
                    "Trying to turn null List of Entitys into Entity[]!");
            }

        Entity[] entityArray = new Entity[entityList.size()];

        return (Entity[]) entityList.toArray(entityArray);
        }

    public String toString()
        {
        return "Entity[" + "[name: " + name + "]" + "[value: " + value + "]" + 
               "]";
        }
    }