// For licensing, see: http://hydrogen.knownspace.org/license.html
//
//VerySimpleEntityManager.java
//Created: 09/28/1999 08:24
///////////////////////////////////////////////////////////////////////////////
//Modified (10/06/2002): maa
//1) Cleaned up JavaDoc to adhere to standards and eliminate conflicts.
//2) Really need to do something major about the massive static blocks.
///////////////////////////////////////////////////////////////////////////////
//Modified (10/08/2002): maa
//1) Replaced Debug.assert()s for null params with IllegalArgumentExceptions.
//2) Logged everywhere that we are throwing an IllegalArgumentException.
//3) Changed Hashtable type declarations to Maps.
///////////////////////////////////////////////////////////////////////////////
// 2002-10-10 cam
// Cleaned up javadoc.
///////////////////////////////////////////////////////////////////////////////
// 2002-10/13 baa
// Finished initial integration with persistence layer and updated javadocs.   
///////////////////////////////////////////////////////////////////////////////
// 2002-10-13 cam/cjd
// Made Serializable.
// Removed defunct finalize code.
///////////////////////////////////////////////////////////////////////////////
package org.datamanager.kernel;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.Serializable;

import java.util.Hashtable;
import java.util.Map;

import org.datamanager.kernel.Persistable;

import org.datamanager.tools.Debug;

/**
 * This is a very simple implementation of an EntityManager.
 * It holds mappings between EntityProxies, Identifiers, and
 * VerySimpleEntityValues.
 * It should be made persistable by the KernelGui's 
 * holds all entities in memory and serializes them to disk
 * when the program is shutdown. Future implementations could
 * do some sort of caching to allow for more entities to be
 * in the system with less RAM required.
 *
 * @author Matthew Farrellee
 * @author Benjamin Keil
 */
class VerySimpleEntityManager
    extends EntityManager
    implements Serializable
    {
    /** String representation of persistence destination (relative path) */
    public static final String PERSISTENCE_FILE_LOCATION = 
            Persistable.PERSISTENCE_FOLDER + File.separator + 
            "VerySimpleEntityManager.xml";
    /** Kernel-level file handle representing persistence destination. */
    static final File PERSISTENCE_FILE = new File(PERSISTENCE_FILE_LOCATION);
    /** The only instance of the VerySimpleEntityManager. */
    private static EntityManager instance = createInstance();
    /** The only instance of the Pool.  Made non-static for compliance with the
     * standard Serializable interface.  Plus, the future holds multiple Pools
     * for us.  
     */
    protected Pool defaultPool;
    /** A mapping from VerySimpleEntities to Identifiers. */
    protected Map entityToIdentifier;
    /** A mapping from Identifiers to VerySimpleEntities. */
    protected Map identifierToEntity;
    /** A mapping from Identifiers to EntityProxies. */
    protected Map identifierToProxy;
    /** A handle for the IdentifierManager's instance. */
    protected IdentifierManager identifierManager = IdentifierManager.getInstance();

    /**
     * Creates a new VerySimpleEntityManager with no mappings.
     */
    private VerySimpleEntityManager()
        {
        entityToIdentifier = new Hashtable();
        identifierToEntity = new Hashtable();
        identifierToProxy = new Hashtable();

        // createDefaultPool may initialize the Singleton Pool via its own
        // persisted XML file or instantiate a new Pool if the persisted file
        // is not found (or some other restoration error occurs).  
        defaultPool = Pool.createDefaultPool();
        assert defaultPool != null;
        }

    /**
     * Requests the only instance of the VerySimpleEntityManager.  The
     * non-null existence of the instance is guaranteed by loadEntityManager. 
     *
     * @return The instance of the VerySimpleEntityManager.
     */
    protected static EntityManager getInstance()
        {
        if (instance == null)
            {
            instance = createInstance();
            }

        return instance;
        }

    private static final EntityManager createInstance()
        {
        loadEntityManager();

        // sanity check on instance.
        assert instance != null;

        return instance;
        }

    /**
     * This method attempts to restore the Singleton instance of
     * VerySimpleEntityManager from the PERSISTENCE_FILE_LOCATION.  If the
     * restoration fails or the PERSISTENCE_FILE does not exist, we
     * instantiate a new VerySimpleEntityManager. 
     */
    private static final void loadEntityManager()
        {
        if (PERSISTENCE_FILE.exists())
            {
            try
                {
                // restore returns null if it fails.
                instance = (VerySimpleEntityManager) restore(PERSISTENCE_FILE);
                }
            catch (Exception e)
                {
                e.printStackTrace();
                Debug.writeLogMessage(
                        "Unable to restore VerySimpleEntityManager: " + 
                        e.getMessage());
                }
            }

        // if instance is still null (i.e., restore failed or PERSISTENCE_FILE
        // did not exist), instantiate a new VerySimpleEntityManager.
        if (instance == null)
            {
            System.err.println("Failed to restore EntityManager");
            instance = new VerySimpleEntityManager();
            }
        }

    /**
     * Requests the default Pool.
     *
     * @return The default pool.
     */
    protected Pool getPool()
        {
        return defaultPool;
        }

    /**
     * Creates a new Entity with the given name.
     *
     * @param name The name of the new Entity.
     * @return An EntityProxy for the new Entity.
     */
    protected synchronized EntityProxy createEntity(String name)
        {
        VerySimpleEntity entity = new VerySimpleEntity(name);
        Identifier identifier = identifierManager.getIdentifier();
        EntityProxy entityProxy = new EntityProxy(identifier);

        entityToIdentifier.put(entity, identifier);
        identifierToEntity.put(identifier, entity);
        identifierToProxy.put(identifier, entityProxy);

        return entityProxy;
        }

    /**
     * Finds the Identifier for the given EntityProxy.
     *
     * @param proxy The EntityProxy for which to find the Identifier.
     * @return The Identifier for the Entity that the EntityProxy is proxying for.
     */
    protected Identifier findIdentifier(EntityProxy proxy)
        {
        if (proxy == null)
            {
            Debug.writeLogMessage(
                    "Trying to find Identifier for a null EntityProxy!");
            throw new IllegalArgumentException(
                    "Trying to find Identifier for a null EntityProxy!");
            }

        return proxy.getIdentifier();
        }

    /**
     * Finds the Identifier for the given Entity.
     *
     * @param entity The Entity for which to find an Identifier.
     * @return The Indentifier for the specified Entity.
     */
    protected Identifier findIdentifier(Entity entity)
        {
        if (entity == null)
            {
            Debug.writeLogMessage(
                    "Trying to find Identifier for a null Entity!");
            throw new IllegalArgumentException(
                    "Trying to find Identifier for a null Entity!");
            }

        return (Identifier) entityToIdentifier.get(entity);
        }

    /**
     * Finds the EntityProxy for the given Identifier.
     *
     * @param identifier The Identitifer for which to find an EntityProxy.
     * @return The EntityProxy associated with the given Identifier.
     */
    protected EntityProxy findProxy(Identifier identifier)
        {
        if (identifier == null)
            {
            Debug.writeLogMessage(
                    "Trying to find EntityProxy for a null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to find EntityProxy for a null Identifier!");
            }

        return (EntityProxy) identifierToProxy.get(identifier);
        }

    /**
     * Finds the Entity associated with the given Identifier.
     *
     * @param identifier The Identifier for which to find an Entity.
     * @return The Entity associated with the Identifier.
     */
    protected Entity findEntity(Identifier identifier)
        {
        if (identifier == null)
            {
            Debug.writeLogMessage(
                    "Trying to find Entity for a null Identifier!");
            throw new IllegalArgumentException(
                    "Trying to find Entity for a null Identifier!");
            }

        return (Entity) identifierToEntity.get(identifier);
        }

    protected void persist(File outputFile)
        {
        Persistable.PERSISTER.persist(this, outputFile);
        }
    }